using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ZylGPSReceiverMobile;

namespace CSharpDemo
{
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
        }

        private bool m_Cancel;

        // This delegate enables asynchronous calls for setting
        // the text property on a TextBox control.
        private delegate void SetTextCallback(string text);

        private delegate void RefreshPaintBoxCallBack();

        private void Set_textBoxOut(string text)
        {
            try
            {
                // InvokeRequired required compares the thread ID of the
                // calling thread to the thread ID of the creating thread.
                // If these threads are different, it returns true.
                if (this.InvokeRequired)
                {
                    this.BeginInvoke(new SetTextCallback(Set_textBoxOut),
                        new object[] { text });
                    return;
                }

                if (this.textBoxOut.Text.Length > 10000)
                    textBoxOut.Text = String.Empty;
                this.textBoxOut.Text += text;
                this.textBoxOut.SelectionStart = textBoxOut.Text.Length;
                this.textBoxOut.SelectionLength = 0;
                this.textBoxOut.ScrollToCaret();
            }
            catch
            {
                this.textBoxOut.Text = string.Empty;
            }
        }        

        private void Set_textBoxCoord(string text)
        {
            try
            {
                // InvokeRequired required compares the thread ID of the
                // calling thread to the thread ID of the creating thread.
                // If these threads are different, it returns true.
                if (this.InvokeRequired)
                {
                    this.BeginInvoke(new SetTextCallback(Set_textBoxCoord),
                        new object[] { text });
                    return;
                }

                this.textBoxCoord.Text = text;
            }
            catch { }
        }

        private void Set_textBoxParams(string text)
        {
            try
            {
                // InvokeRequired required compares the thread ID of the
                // calling thread to the thread ID of the creating thread.
                // If these threads are different, it returns true.
                if (this.InvokeRequired)
                {
                    this.BeginInvoke(new SetTextCallback(Set_textBoxParams),
                        new object[] { text });
                    return;
                }

                this.textBoxParams.Text = text;
            }
            catch { }
        }

        private void RefreshPaintBox()
        {
            try
            {
                // InvokeRequired required compares the thread ID of the
                // calling thread to the thread ID of the creating thread.
                // If these threads are different, it returns true.
                if (this.InvokeRequired)
                {
                    this.BeginInvoke(new RefreshPaintBoxCallBack(RefreshPaintBox));
                    return;
                }

                this.pBox.Refresh();
            }
            catch { }
        }


        private void btnClose_Click(object sender, EventArgs e)
        {
            cmpGpsReceiver.Close();            
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            if (lstPorts.SelectedIndex >= 0 && lstBaud.SelectedIndex >= 0)
            {
                cmpGpsReceiver.Port = (SerialCommPort)(lstPorts.SelectedIndex + 1);
                cmpGpsReceiver.BaudRate = (SerialBaudRate)(Convert.ToUInt32(lstBaud.SelectedItem.ToString()));
                if (!cmpGpsReceiver.Open())
                    MessageBox.Show("Cannot open.");                
            }
            else
            {
                MessageBox.Show("Select port and baud rate.");
            }
        }

        private void cmpGpsReceiver_Received(object sender, ZylGPSReceiverMobile.DataEventArgs e)
        {
            string str;
            str = "Lt: " + cmpGpsReceiver.Position.LatitudeDegree.ToString() + "" +
                cmpGpsReceiver.Position.LatitudeMinute.ToString() + "'" +
                cmpGpsReceiver.Position.LatitudeSecond.ToString("N2") + "''";
            if (cmpGpsReceiver.Position.LatitudeDirection == GPSPosition.CardinalPoint.North)
                str += "N ";
            else
                str += "S ";
            str += "Ln: " + cmpGpsReceiver.Position.LongitudeDegree.ToString() + "" +
                cmpGpsReceiver.Position.LongitudeMinute.ToString() + "'" +
                cmpGpsReceiver.Position.LongitudeSecond.ToString("N2") + "''";
            if (cmpGpsReceiver.Position.LongitudeDirection == GPSPosition.CardinalPoint.East)
                str += "E ";
            else
                str += "W ";            
            Set_textBoxCoord(str);
            str = "Alt: " + cmpGpsReceiver.Position.Altitude.ToString("N2") + " m ";
            str += "Speed: " + cmpGpsReceiver.Position.SpeedAsKmh.ToString("N2") + " km/h ";            
            Set_textBoxParams(str);
            Set_textBoxOut(e.Buffer + "\r\n");    
        }

        private void cmpGpsReceiver_Connected(object sender, ZylGPSReceiverMobile.ConnectionEventArgs e)
        {
            Set_textBoxOut("Connected to: " + e.Port.ToString());
        }

        private void cmpGpsReceiver_Disconnected(object sender, ZylGPSReceiverMobile.ConnectionEventArgs e)
        {
            Set_textBoxOut("Disconnected from: " + e.Port.ToString());
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void FormMain_Closed(object sender, EventArgs e)
        {
            if (cmpGpsReceiver.ConnectedTo != SerialCommPort.None)
                cmpGpsReceiver.Close();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            lstBaud.SelectedIndex = 5;
        }

        private void cmpGpsReceiver_SatellitesReceived(object sender, EventArgs e)
        {
            RefreshPaintBox();          
        }

        private void pBox_Paint(object sender, PaintEventArgs e)
        {
            Bitmap b = new Bitmap(pBox.Width, pBox.Width);
            Graphics g = Graphics.FromImage(b);
            cmpGpsReceiver.DrawSatellites(g, pBox.Width/2, Color.Black, Color.White);
            e.Graphics.DrawImage(b, 0, 0);
            g.Dispose();
            b.Dispose();            
        }

        private void btnDetect_Click(object sender, EventArgs e)
        {
            SerialCommPort port;
            SerialBaudRate baudRate;
            lblDetect.Text = "";
            m_Cancel = false;
            if (cmpGpsReceiver.FastDetectGPS(out port, out baudRate))
            {
                MessageBox.Show("Port: " + GPSReceiver.SerialCommPortToString(port) + ", Baud Rate: " + baudRate.ToString());
                lstPorts.SelectedIndex = lstPorts.Items.IndexOf(GPSReceiver.SerialCommPortToString(port));
                lstBaud.SelectedIndex = lstBaud.Items.IndexOf(((int)baudRate).ToString());
            }
            else
                MessageBox.Show("No GPS detected.");
            lblDetect.Text = "";

        }

        private void cmpGpsReceiver_Detecting(object sender, DetectEventArgs e, ref bool Cancel)
        {
            lblDetect.Text = "Detecting - Port: " + GPSReceiver.SerialCommPortToString(e.Port) +
                ", Baud Rate: " + e.BaudRate.ToString();
            Cancel = m_Cancel;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            m_Cancel = true;
        }

        private void cmpGpsReceiver_Position2DChanged(object sender, DMSPositionEventArgs e)
        {
            Set_textBoxOut("Position2D changed.");
        }        
    }
}