Imports ZylGPSReceiverMobile

Public Class FormMain

    ' This delegate enables asynchronous calls for setting
    ' the text property on a TextBox control.
    Private Delegate Sub SetTextCallback(ByVal text As String)
    Private Delegate Sub RefreshPaintBoxCallBack()

    Private Sub SetTextBoxOut(ByVal text As String)
        Try

            ' InvokeRequired required compares the thread ID of the
            ' calling thread to the thread ID of the creating thread.
            ' If these threads are different, it returns true.
            If Me.InvokeRequired Then
                Dim d As New SetTextCallback(AddressOf SetTextBoxOut)
                Me.BeginInvoke(d, New Object() {[text]})
                Return
            End If

            If Me.textBoxOut.Text.Length > 10000 Then
                textBoxOut.Text = String.Empty
            End If

            Me.textBoxOut.Text &= text
            Me.textBoxOut.SelectionStart = textBoxOut.Text.Length
            Me.textBoxOut.SelectionLength = 0
            Me.textBoxOut.ScrollToCaret()
        Catch
            Me.textBoxOut.Text = String.Empty
        End Try
    End Sub

    Private Sub SetTextBoxCoord(ByVal text As String)
        Try
            ' InvokeRequired required compares the thread ID of the
            ' calling thread to the thread ID of the creating thread.
            ' If these threads are different, it returns true.
            If (Me.InvokeRequired) Then
                Dim d As New SetTextCallback(AddressOf SetTextBoxCoord)

                Me.BeginInvoke(d, New Object() {[text]})
                Return

            End If
            If Me.textBoxOut.Text.Length > 10000 Then
                textBoxOut.Text = String.Empty
            End If
            Me.textBoxCoord.Text = text
        Catch
        End Try
    End Sub

    Private Sub SetTextBoxParams(ByVal text As String)
        Try
            ' InvokeRequired required compares the thread ID of the
            ' calling thread to the thread ID of the creating thread.
            ' If these threads are different, it returns true.
            If (Me.InvokeRequired) Then
                Dim d As New SetTextCallback(AddressOf SetTextBoxParams)
                Me.BeginInvoke(d, New Object() {[text]})
                Return

            End If
            Me.textBoxParams.Text = text
        Catch
        End Try
    End Sub

    Private Sub RefreshPaintBox()        
        Try
            If (Me.InvokeRequired) Then

                Dim d As New RefreshPaintBoxCallBack(AddressOf RefreshPaintBox)
                Me.BeginInvoke(d)
                Return
            End If

            Me.pBox.Refresh()
        Catch
        End Try

    End Sub

    Private Sub btnOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOpen.Click
        If lstPorts.SelectedIndex >= 0 AndAlso lstBaud.SelectedIndex >= 0 Then
            cmpGpsReceiver.Port = CType(lstPorts.SelectedIndex + 1, SerialCommPort)
            cmpGpsReceiver.BaudRate = CType(Convert.ToUInt32(lstBaud.SelectedItem.ToString()), SerialBaudRate)
            If Not cmpGpsReceiver.Open() Then
                MessageBox.Show("Cannot open.")
            End If
        Else
            MessageBox.Show("Select port and baud rate.")
        End If
    End Sub
    Private Sub btnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClose.Click
        cmpGpsReceiver.Close()
    End Sub

    Private Sub cmpGpsReceiver_Received(ByVal sender As System.Object, ByVal e As ZylGPSReceiverMobile.DataEventArgs) Handles cmpGpsReceiver.Received
        Dim str As String
        str = "Lt: " & cmpGpsReceiver.Position.LatitudeDegree.ToString() & "" & _
                cmpGpsReceiver.Position.LatitudeMinute.ToString() & "'" & _
                cmpGpsReceiver.Position.LatitudeSecond.ToString("N2") & "''"
        If cmpGpsReceiver.Position.LatitudeDirection = GPSPosition.CardinalPoint.North Then
            str = str & "N "
        Else
            str = str & "S "
        End If
        str = str & "Ln: " + cmpGpsReceiver.Position.LongitudeDegree.ToString() & "" & _
            cmpGpsReceiver.Position.LongitudeMinute.ToString() & "'" & _
            cmpGpsReceiver.Position.LongitudeSecond.ToString("N2") & "''"
        If cmpGpsReceiver.Position.LongitudeDirection = GPSPosition.CardinalPoint.East Then
            str = str & "E "
        Else
            str = str & "W "
        End If
        SetTextBoxCoord(str)
        str = "Alt: " & cmpGpsReceiver.Position.Altitude.ToString("N2") & " m "
        str = str & "Speed: " & cmpGpsReceiver.Position.SpeedAsKmh.ToString("N2") & " km/h"
        SetTextBoxParams(str)
        SetTextBoxOut(e.Buffer)
    End Sub

    Private Sub cmpGpsReceiver_Connected(ByVal sender As System.Object, ByVal e As ZylGPSReceiverMobile.ConnectionEventArgs) Handles cmpGpsReceiver.Connected
        SetTextBoxOut("Connected to: " & e.Port.ToString())
    End Sub

    Private Sub cmpGpsReceiver_Disconnected(ByVal sender As System.Object, ByVal e As ZylGPSReceiverMobile.ConnectionEventArgs) Handles cmpGpsReceiver.Disconnected
        SetTextBoxOut("Disconnected from: " & e.Port.ToString())
    End Sub

    Private Sub FormMain_Closed(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        If cmpGpsReceiver.ConnectedTo <> SerialCommPort.None Then
            cmpGpsReceiver.Close()
        End If
    End Sub

    Private Sub btnExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnExit.Click
        Application.Exit()
    End Sub

    Private Sub FormMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        lstBaud.SelectedIndex = 5
    End Sub

    Private Sub cmpGpsReceiver_SatellitesReceived(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmpGpsReceiver.SatellitesReceived
        RefreshPaintBox()
    End Sub

    Private Sub pBox_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles pBox.Paint
        Dim b As Bitmap = New Bitmap(pBox.Width, pBox.Width)
        Dim g As Graphics = Graphics.FromImage(b)
        cmpGpsReceiver.DrawSatellites(g, pBox.Width / 2, Color.Black, Color.White)
        e.Graphics.DrawImage(b, 0, 0)
        g.Dispose()
        b.Dispose()
    End Sub

    Private Sub btnDetect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDetect.Click
        Dim port As SerialCommPort
        Dim baudRate As SerialBaudRate
        If cmpGpsReceiver.FastDetectGPS(port, baudRate) Then
            MessageBox.Show("Port: " & GPSReceiver.SerialCommPortToString(port) & ", Baud Rate: " & baudRate.ToString())
            lstPorts.SelectedIndex = lstPorts.Items.IndexOf(GPSReceiver.SerialCommPortToString(port))
            lstBaud.SelectedIndex = lstBaud.Items.IndexOf(Convert.ToInt32(baudRate).ToString())
        Else
            MessageBox.Show("No GPS detected.")
        End If

    End Sub
End Class
